#!/usr/bin/python3
"""
// © University of Southampton IT Innovation Centre, 2018
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road,
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Nikolay Stanchev
//      Created Date :          16-08-2018
//      Created for Project :   FLAME
"""


# Python standard libs
from os.path import join

# PIP installed libs
from schema import SchemaError

# CLMC-service imports
from clmcservice import ROOT_DIR
from clmcservice.alertsapi.alerts_specification_schema import ALERTS_SPECIFICATION_SCHEMA


CLMC_ALERTS_TOSCA_DEFINITIONS_REL_PATH = ["resources", "tosca", "flame_clmc_alerts_definitions.yaml"]

CLMC_ALERTS_TOSCA_DEFINITIONS_ABS_PATH = join(ROOT_DIR, *CLMC_ALERTS_TOSCA_DEFINITIONS_REL_PATH)

CLMC_ALERTS_TOSCA_DEFINITIONS_FILE = CLMC_ALERTS_TOSCA_DEFINITIONS_REL_PATH[-1]


def adjust_tosca_definitions_import(alert_spec):
    """
    A utility function to adjust any imports of flame_clmc_alerts_definitions.yaml to point to the correct location of
    the tosca definitions file.

    :param alert_spec: the TOSCA alert specification content (yaml dict)
    """

    global CLMC_ALERTS_TOSCA_DEFINITIONS_ABS_PATH

    try:
        import_index = alert_spec["imports"].index(CLMC_ALERTS_TOSCA_DEFINITIONS_FILE)
        alert_spec["imports"][import_index] = CLMC_ALERTS_TOSCA_DEFINITIONS_ABS_PATH
    except Exception:
        pass  # nothing to replace if the import is not specified (either imports are missed, or no reference to the clmc tosca definitions file)


def validate_clmc_alerts_specification(tosca_yaml_tpl, include_error=False):
    """
    CLMC validation for the TOSCA alerts specification, uses the schema defined in alerts_specification_schema.py

    :param tosca_yaml_tpl: the tosca template to validate (as python dictionary object)
    :param include_error: a flag indicating whether the output of the function should include a caught SchemaError
        (if set to True and no error is thrown, returns None as the error object)

    :return: True/False if the tosca_tpl is valid/invalid along with any error (None if no error) that was thrown during validation (if argument include_error is set to True)
    """

    try:
        ALERTS_SPECIFICATION_SCHEMA.validate(tosca_yaml_tpl)
        valid, err = True, None
    except SchemaError as e:
        valid, err = False, e

    if include_error:
        return valid, err
    else:
        return valid
